searchNodes=[{"doc":"A simple composable transaction library that serves as an experimental alternative solution to Ecto.Multi. This library intend to tackle the main problem with Ecto.Multi: Ecto.Multi names are global, which requires the caller of a multi-function to know about the names used in the multi. Composing complex Ecto.Multi can run into name collision if not being careful enough.","ref":"Tx.html","title":"Tx","type":"module"},{"doc":"Compose two transactions. Run the second one only if the first one succeeds. This is the &quot;bind&quot; operation if you're familiar with Monad. Example: iex&gt; Tx . pure ( 1 ) |&gt; Tx . and_then ( &amp; { :error , &amp;1 } ) |&gt; Tx . execute ( Repo ) { :error , 1 }","ref":"Tx.html#and_then/2","title":"Tx.and_then/2","type":"function"},{"doc":"Combine a list of transactions [Tx.t(a)] into a single Tx.t([a]) . iex&gt; Tx . concat ( [ Tx . pure ( 1 ) , Tx . pure ( 2 ) ] ) |&gt; Tx . execute ( Repo ) { :ok , [ 1 , 2 ] }","ref":"Tx.html#concat/1","title":"Tx.concat/1","type":"function"},{"doc":"Combine two transactions Tx.t(a) and Tx.t(b) into a single Tx.t({a, b}) . iex&gt; Tx . concat ( Tx . pure ( 1 ) , Tx . pure ( 2 ) ) |&gt; Tx . execute ( Repo ) { :ok , { 1 , 2 } }","ref":"Tx.html#concat/2","title":"Tx.concat/2","type":"function"},{"doc":"Avoid a transaction to rollback on exception. Wrap around tx with rollback_on_exception(tx, false) if you avoid tx to rollback on exception. When an exception occurs, you will get an {:error, exception} instead.","ref":"Tx.html#disable_rollback_on_exception/1","title":"Tx.disable_rollback_on_exception/1","type":"function"},{"doc":"Make an transaction rollback on error. You can use this function to fine-tune the rollback behaviour on specific sub-transactions.","ref":"Tx.html#enable_rollback_on_error/1","title":"Tx.enable_rollback_on_error/1","type":"function"},{"doc":"Execute the transaction Tx.t(a) , producing an {:ok, a} or {:error, any} . Options: rollback_on_error (Default: true): rollback transaction if the final result is an {:error, any() rollback_on_exception (Default: true): rollback transaction if an uncaught exception arises within the transaction. Any options Ecto.Repo.transaction/2 accepts.","ref":"Tx.html#execute/3","title":"Tx.execute/3","type":"function"},{"doc":"Map over the successful result of a transaction. Example: iex&gt; Tx . pure ( 1 ) |&gt; Tx . map ( &amp; ( &amp;1 + 1 ) ) |&gt; Tx . execute ( Repo ) { :ok , 2 }","ref":"Tx.html#map/2","title":"Tx.map/2","type":"function"},{"doc":"Create a transaction. This function creates a Tx.t() from a function of type (Ecto.Repo.t() -&gt; {:ok, a} | {:error, any()}) . Internally, the new/1 constructor simply returns the function as is. Example: iex&gt; t = fn _repo -&gt; { :ok , 42 } end iex&gt; Tx . execute ( Tx . new ( t ) , Repo ) { :ok , 42 }","ref":"Tx.html#new/1","title":"Tx.new/1","type":"function"},{"doc":"Create a transaction that returns a pure value. This is the &quot;pure&quot;/&quot;return&quot; operation if you're familiar with Monad. Executing Tx with a pure value always returns the value. pure(a) is equivalent to new(fn _ -&gt; {:ok, a} end) . iex&gt; Tx . execute ( Tx . pure ( 42 ) , Repo ) { :ok , 42 }","ref":"Tx.html#pure/1","title":"Tx.pure/1","type":"function"},{"doc":"Rollback the current transaction. Example: iex&gt; Tx . new ( fn repo -&gt; ...&gt; if 1 == 1 do ...&gt; Tx . rollback ( repo , &quot;One cannot be equal to one&quot; ) ...&gt; else ...&gt; { :ok , :fine } ...&gt; end ...&gt; end ) ...&gt; |&gt; Tx . execute ( Repo ) { :error , &quot;One cannot be equal to one&quot; }","ref":"Tx.html#rollback/2","title":"Tx.rollback/2","type":"function"},{"doc":"The raising version of run/2 . This function should be rarely needed if you use the Tx.Macro.tx macro.","ref":"Tx.html#run!/2","title":"Tx.run!/2","type":"function"},{"doc":"Run a transaction to get its result. This is a generic adapter for extracting a {:ok, a} | {:error, any()} from a transaction when given a repo. For normal transaction tx as a function (default), it simply call tx.(repo) For Ecto.Multi, it creates a sub-transaction to execute it For a non-transactional value, it simply returns the value This function should be rarely needed if you use the Tx.Macro.tx macro. You can simply use a &lt;- t syntax instead of a = Tx.run(repo, t) within the tx macro. This function is meant to be used within a tx block or inside a Tx closure. If you want to get a value out of a Tx, you may want to call execute/2 instead.","ref":"Tx.html#run/2","title":"Tx.run/2","type":"function"},{"doc":"Convert a Tx.t(a) into a Multi.t() . You can refer to the transactin's result ( {:ok, a} | {:error, any} ) by name . EctoRepo . transaction ( to_multi ( pure ( 42 ) , :foo ) ) =&gt; { :ok , %{ foo : { :ok , 42 } } }","ref":"Tx.html#to_multi/2","title":"Tx.to_multi/2","type":"function"},{"doc":"","ref":"Tx.html#t:fn_t/1","title":"Tx.fn_t/1","type":"type"},{"doc":"","ref":"Tx.html#t:t/0","title":"Tx.t/0","type":"type"},{"doc":"You can put a value of any Tx type on the right of &lt;- notation in Tx.Macro. A Tx type is any of the following: A Tx function :: any function that takes in a repo and returns a {:ok, a} | {:error, any()} pair. You can create a Tx function via the tx macro, or any tx combinators like Tx.new , Tx.pure , Tx.concat . An Ecto.Multi.t() :: equivalent a tx function that returns {:ok, %{multi_name =&gt; value}} or {:error, multi_error} nil :: the same was as Tx.pure(nil) , which always returns {:ok, nil} when been run. A list of any Tx type :: the same as Tx.concat(list_of_tx) , which returns {:ok, list_of_results} only if all list_of_tx succeeds.","ref":"Tx.html#t:t/1","title":"Tx.t/1","type":"type"},{"doc":"Export macros to allow create complex transactions without boilerplate.","ref":"Tx.Macro.html","title":"Tx.Macro","type":"module"},{"doc":"Create a transaction. import Tx.Macro tx do { :ok , a } &lt;- create_a_tx ( value ) { :ok , b } &lt;- create_b_tx ( a ) { :ok , { a , b } } end will expands into: Tx . new ( fn repo -&gt; with { :ok , a } &lt;- Tx . run ( repo , create_a_tx ( value ) ) , { :ok , b } &lt;- Tx . run ( repo , create_b_tx ( a ) ) do { :ok , { a , b } } end end ) You can use tx/2 if you need to access to a binding to repo from the transaction.","ref":"Tx.Macro.html#tx/1","title":"Tx.Macro.tx/1","type":"macro"},{"doc":"Create a transaction with a binding to repo . The repo binding can be used within the body for raw db operations (e.g. Repo.insert , Repo.update , ...). Example: The following code: import Tx.Macro tx repo do { :ok , value } &lt;- repo . insert ( foo ) { :ok , a } &lt;- create_a_tx ( value ) { :ok , b } &lt;- create_b_tx ( a ) { :ok , { a , b } } end will expands into: Tx . new ( fn repo -&gt; with { :ok , value } &lt;- Tx . run ( repo , repo . insert ( foo ) ) , { :ok , a } &lt;- Tx . run ( repo , create_a_tx ( value ) ) , { :ok , b } &lt;- Tx . run ( repo , create_b_tx ( a ) ) do { :ok , { a , b } } end end )","ref":"Tx.Macro.html#tx/2","title":"Tx.Macro.tx/2","type":"macro"}]